@@ -3,7 +3,16 @@ This module provides an interface to the Berkeley EMD file format.
 
 See https://emdatasets.com/ for more details.
 
+* It fully supports v0.2 for reading and writing
 
+Note
+----
+General users:
+    Use the simplified emd.emdReader() function to load the data and meta
+    data as a python dictionary.
+
+Advanced users and developers:
+    Access the file internals through the emd.fileDEMD() class.
 
 """
 
@@ -15,38 +24,51 @@ import h5py
 
 
 class fileEMD:
-    """Class to represent EMD files.
+    """Class to represent Berkelely EMD files.
 
     Implemented for spec 0.2 using the recommended layout for metadata.
 
-    Meant to provide convenience functions for commonly occurring tasks. This means that you will still want to access
-     fileEMD.file_hdl to manipulate the HDF5 file for not so commonly occurring tasks.
-
-    Parameters
+    Attributes
     ----------
-        filename : str
-            Name of the EMD file.
-        readonly : bool, default True
-            Set to False to allow writing to the file.
 
-    Example
-    -------
-        Open an Berkeley EMD file. List the available data sets. Load a 3D data set and plot the first image.
+    file_hdl : h5py.File
+        The h5py file handle which provides direct access to the underlying h5py file structure.
+    version : tuple
+        A 2-tuple of the major and minor EMD version as ints.
+    data : h5py.Group
+        A link to the data group in the EMD file.
+    microscope : h5py.Group
+        A link to the microsocpe EMD group with metadata.
+    sample : h5py.Group
+        A link to the sample EMD group with metadata.
+    user : h5py.Group
+        A link to the user EMD group with metadata.
+    comments : h5py.Group
+        A link to the comments in this EMD file. A time stamp is included with the comment.
+    list_emds : list
+        A list of the EMD groups in the file.
 
-            >>> from ncempy.io import emd
-            >>> import matplotlib.pyplot as plt
-            >>> emd1 = emd.fileEMD('filename.emd')
-            >>> [print(dataGroup.name) for dataGroup in emd1.list_emds] # Use the builtin list_emds variable to print all available EMD datasets
-            >>> data1,dims1 = emd1.get_emdgroup(emd1.list_emds[0]) # load the first full data array and dimension information
-            >>> fg1,ax1 = plt.subplots(1,1)
-            >>> ax1.imshow(data1[0,:,:],extent=(dims1[1][0][0],dims1[1][0][-1],dims1[2][0][0],dims1[2][0][-1])) # the extent uses the first and last array values of the dimension vectors
-            >>> ax1.set(xlabel='{0[1]} ({0[2]})'.format(dims1[1]),ylabel='{0[1]} ({0[2]})'.format(dims1[2])) # label the axes with the name and units of each dimension vector
-            >>> del emd1 #close the emd file
+    Examples
+    --------
+    Open an Berkeley EMD file using the *advanced* interface. See the emdReader function for a more convenient
+    way to load the data. We show how to list the available data sets, load a 3D data set and plot the first image.
+
+        >>> from ncempy.io import emd
+        >>> with emd.fileEMD('filename.emd') as emd1:
+        >>>     [print(dataGroup.name) for dataGroup in emd1.list_emds] # print all available EMD datasets
+        >>>     data1, dims1 = emd1.get_emdgroup(0) # load the first full data array and dimension information
     """
 
-    def __init__(self, filename, readonly=False):
+    def __init__(self, filename, readonly=True):
         """Init opening/creating the file.
 
+        Parameters
+        ----------
+        filename : str or pathlib.Path
+            Path to the EMD file.
+        readonly : bool, default True
+            Set to False to allow writing to the file.
+
         """
 
         # necessary declarations in case something goes bad
@@ -59,9 +81,9 @@ class fileEMD:
         self.sample = None
         self.user = None
         self.comments = None
-        self.list_emds = []  # list of HDF5 groups with emd_data_type type
+        self.list_emds = []  # list of HDF5 groups with emd_data_type type 1
 
-        # check filename type
+        # check filename type, change to string
         if isinstance(filename, str):
             pass
         elif isinstance(filename, Path):
@@ -91,9 +113,8 @@ class fileEMD:
                 self.version = (self.file_hdl.attrs['version_major'], self.file_hdl.attrs['version_minor'])
                 # compare to implementation
                 if not self.version == (0, 2):
-                    print(
-                        'WARNING: You are reading a version {}.{} EMD file, this implementation assumes version 0.2!'.format(
-                            self.version[0], self.version[1]))
+                    print('WARNING: You are reading a version {}.{} EMD file, '
+                          'this implementation assumes version 0.2!'.format(self.version[0], self.version[1]))
             else:
                 # set version information
                 if not readonly:
@@ -164,44 +185,44 @@ class fileEMD:
 
         Parameters
         ----------
-            parent: h5py._hl.group.Group
+            parent: h5py.Group
                 Handle to the parent group.
 
         Returns
         -------
             : list
-                A list of h5py._hl.group.Group handles to children groups
+                A list of h5py.Group handles to children groups
                 being emd_data_type groups.
 
         """
 
-        emds = []
+        emds0 = []
 
         # recursive function to run and retrieve groups with emd_group_type set to 1
         def proc_group(group, emds):
             # take a look at each item in the group
             for item in group:
                 # check if group
-                if group.get(item, getclass=True) == h5py._hl.group.Group:
+                if group.get(item, getclass=True) == h5py.Group:
                     item = group.get(item)
                     # check if emd_group_type
-                    if 'emd_group_type' in item.attrs:
+                    if 'emd_group_type' in item.attrs and 'data' in item.keys():
                         if item.attrs['emd_group_type'] == 1:
                             emds.append(item)
                     # process subgroups
                     proc_group(item, emds)
 
         # run
-        proc_group(parent, emds)
+        proc_group(parent, emds0)
 
-        return emds
+        return emds0
 
     def get_emddims(self, group):
         """Get the emdtype dimensions saved in in group.
 
         Parameters
         ----------
-            group: h5py._hl.group.Group
+            group: h5py.Group
                 Reference to the emdtype HDF5 group.
 
         Returns
@@ -212,8 +233,8 @@ class fileEMD:
         """
         # get the dims
         dims = []
-        for i in range(len(group['data'].shape)):
-            dim = group['dim{}'.format(i + 1)]
+        for ii in range(group['data'].ndim):
+            dim = group['dim{}'.format(ii + 1)]
             # save them as (vector, name, units)
 
             if isinstance(dim.attrs['name'], np.ndarray):
@@ -226,12 +247,22 @@ class fileEMD:
             else:
                 units = dim.attrs['units']
 
-            dims.append((dim[:], name.decode('utf-8'), units.decode('utf-8')))
+            # Handle bytes objects by decoding them to strings
+            # If something goes wrong, the original attribute is left as-is
+            try:
+                if isinstance(name, bytes):
+                    name = name.decode('utf-8')
+                if isinstance(units, bytes):
+                    units = units.decode('utf-8')
+            except:
+                pass
+
+            dims.append((dim[:], name, units))
 
         dims = tuple(dims)
-        return (dims)
+        return dims
 
-    def get_emdgroup(self, group, memmap = False):
+    def get_emdgroup(self, group, memmap=False):
         """Get the emd data saved in the requested group.
 
         Parameters
@@ -244,19 +275,16 @@ class fileEMD:
 
         Returns
         -------
-            : tuple/None
+            : tuple or None
                 None or tuple containing:
-
-                : np.ndarray
-                    The data of the emdtype group.
-
-                : list
-                    List of [0] dimension vectors, [1] labels and [2] units.
-
+                    : np.ndarray
+                        The data of the emdtype group.
+                    : list
+                        List of [0] dimension vectors, [1] labels and [2] units.
         """
 
         # check input
-        if not isinstance(group, h5py._hl.group.Group):
+        if not isinstance(group, h5py.Group):
             if isinstance(group, int):
                 try:
                     group = self.list_emds[group]
@@ -266,7 +294,7 @@ class fileEMD:
             else:
                 raise TypeError('group needs to refer to a valid HDF5 group!')
 
-        if not 'emd_group_type' in group.attrs:
+        if 'emd_group_type' not in group.attrs:
             raise TypeError('group is not a emd_group_type group!')
         if not group.attrs['emd_group_type'] == 1:
             raise TypeError('group is not a emd_group_type group!')
@@ -283,7 +311,6 @@ class fileEMD:
             dims = self.get_emddims(group)
 
             return data, dims
-
         except:
             # if something goes wrong, return None
             print('Content of "{}" does not seem to be in emd specified shape'.format(group.name))
@@ -301,12 +328,12 @@ class fileEMD:
                 Label for dataset, usually dim1, dim2, dimN.
             dim: tuple
                 Tuple containing (data, name, units).
-            parent: h5py._hl.group.Group
+            parent: h5py.Group
                 HDF5 handle to parent group.
 
         Returns
         -------
-            : h5py._hl.group.Group
+            : h5py.Group
                 HDF5 dataset handle referencing this dim.
 
         """
@@ -331,7 +358,7 @@ class fileEMD:
                 Numpy array containing the data.
             dims: tuple
                 Tuple containing the necessary dims as ((vec, name, units), (vec, name, units), ...)
-            parent: h5py._hl.group.Group/None
+            parent: h5py.Group or None
                 Parent for the emdtype group, if None it will be written to /data.
             overwrite: bool
                 Set to force overwriting entry in EMD file.
@@ -340,7 +367,7 @@ class fileEMD:
 
         Returns
         -------
-            : h5py._hl.group.Group/None
+            : h5py.Group or None
                 Group referencing this emdtype dataset or None if failed.
         """
 
@@ -444,13 +471,13 @@ class fileEMD:
     def get_memmap(self, group):
         """ Get the emd group data as a memmap so that the data
         is not loaded into memory. Essentially calls get_emdgroup()
-        with the keyword memmap keyord equals True.
+        with the keyword memmap keyword equals True.
         
         See get_emdgroup() for parameters and return values.
         
         
         """
-        return self.get_emdgroup(group, memmap = True)
+        return self.get_emdgroup(group, memmap=True)
         
 
 def defaultDims(data, pixel_size=None):
@@ -519,8 +546,8 @@ def emdReader(filename, dsetNum=0):
             >> emd0 = nio.emd.emdReader('filename.emd', dsetNum = 0)
 
     """
-    with fileEMD(filename, readonly = True) as emd0:
-        d, dims = emd0.get_emdgroup(dsetNum, memmap = False) # memmap must be false. File is closed
+    with fileEMD(filename, readonly=True) as emd0:
+        d, dims = emd0.get_emdgroup(dsetNum, memmap=False)  # memmap must be false. File is closed
         out = {'data': d, 'filename': filename, 'pixelSize': []}
 
         for dim in dims:
@@ -534,12 +561,34 @@ def emdReader(filename, dsetNum=0):
         return out
 
 
-if __name__ == '__main__':
-    fPath = Path(r'C:\Users\linol\Data') / Path('Acquisition_18.emd')
+def emdWriter(filename, data, pixel_size=None):
+    """ Simple method to write data to a file formatted as an EMD v0.2. The only possible metadata to write is the pixel
+    size for each dimension. Use the emd.fileEMD() class for more complex operations. The file must not already exist.
 
-    emd00 = emdReader(fPath)
+    Parameters
+    ----------
+    filename : str or pathlib.Path
+        The path and file name to write the data to.
+    data : ndarray
+        The data as an ndarray to write to the file.
+    pixel_size : tuple
+        A tuple with the same length as the number of dims in data. len(pixel_size) == data.ndim
 
-    print(emd00['pixelSize'])
+    """
+    if isinstance(filename, str):
+        filename = Path(filename)
 
-    print(defaultDims(np.zeros((10, 20, 30)), pixel_size=(0.1, 0.2, 0.3)))
-    print(defaultDims(np.zeros((10, 20, 30))))
\ No newline at end of file
+    if pixel_size:
+        try:
+            assert len(pixel_size) == data.ndim
+        except ValueError:
+            raise ValueError('pixel_size length must match the number of dimensions of data.')
+
+    if not filename.exists():
+        with fileEMD(filename, readonly=False) as emd0:
+            # Setup the dims for the EMD file. Pixel size is set to 1 by default for each dimension
+            dims0 = defaultDims(data, pixel_size=pixel_size)
+            # Write the data to he emd file.
+            emd0.put_emdgroup('converted', data, dims0)
+    else:
+        raise FileExistsError
